#ifdef HAVE_IMLIB
# include <Imlib.h>
#endif
#ifdef HAVE_IMLIB2
# include <X11/Xlib.h>
# include <Imlib2.h>
#endif
#include <gtk/gtk.h>

#include "imgview.h"
#include "ivtitle.h"


#ifdef INCLUDE_IV_TITLE_IMAGE
#include "iv_title_320x240.xpm"
#endif


#ifdef HAVE_IMLIB
/* Handle used for passing to all Imlib functions, not needed for
 * Imlib2. This is foundationally declared in main.c.
 */
extern void *imlib_handle;
#endif  /* HAVE_IMLIB */


void iv_load_title(imgview_struct *iv, gbool resize_on_load);


/*
 *	Loads the title image from data into the given viewer.
 */
void iv_load_title(imgview_struct *iv, gbool resize_on_load)
{
#ifdef INCLUDE_IV_TITLE_IMAGE
	GtkWidget *w;

#ifdef HAVE_IMLIB
	ImlibImage *imlib_image;
#endif
#ifdef HAVE_IMLIB2
        Imlib_Image imlib_image;
#endif


	if(iv == NULL)
	    return;

        /* Get image viewer toplevel widget if it is a GtkWindow. */
        if(ImgViewToplevelIsWindow(iv))
            w = ImgViewGetToplevelWidget(iv);
        else
            w = NULL;

#ifdef HAVE_IMLIB
	if(imlib_handle != NULL)
	{
	    /* Load title image. */
	    imlib_image = Imlib_create_image_from_xpm_data(
		imlib_handle, (char **)iv_title_320x240_xpm
	    );
	    if(imlib_image != NULL)
	    {
                const guint8 *src_data_rgb, *src_data_alpha;
		guint8 *data = NULL;
		gint width, height;


		/* Need to realize changes. */
		Imlib_changed_image(imlib_handle, imlib_image);

		/* Get pointer to buffers and size of newly loaded image. */
		src_data_rgb = (const guint8 *)imlib_image->rgb_data;
		src_data_alpha = (const guint8 *)imlib_image->alpha_data;
		width = imlib_image->rgb_width;
		height = imlib_image->rgb_height;

		/* Loaded Imlib image data valid? */
		if((src_data_rgb != NULL) && (width > 0) && (height > 0))
		{
		    gint tar_bpl, src_bpl_rgb, src_bpl_alpha;
		    GtkWidget *w2 = (GtkWidget *)ImgViewGetViewWidget(iv);


                    /* Calculate bytes per line values. */
                    /* Local image buffer bytes per line. */
                    tar_bpl = width * sizeof(guint32);
                    /* Source image rgb buffer bytes per line. */
                    src_bpl_rgb = width * (3 * sizeof(guint8));
                    /* Source image alpha buffer bytes per line. */
                    src_bpl_alpha = width * (1 * sizeof(guint8));

                    /* Allocate local image data buffer. */
                    data = (guint8 *)g_realloc(
                        data,
                        tar_bpl * height
                    );
                    if(data != NULL)
                    {
                        gint x, y;
                        guint8 *tar_ptr;
                        const guint8 *src_ptr;

                        for(y = 0; y < height; y++)
                        {
                            for(x = 0; x < width; x++)
                            {
                                tar_ptr = &data[
                                    (y * tar_bpl) +
                                    (x * sizeof(guint32))
                                ];
                                src_ptr = &src_data_rgb[
                                    (y * src_bpl_rgb) +
                                    (x * (3 * sizeof(guint8)))
                                ];

                                /* Copy rgb of this pixel (3 bytes). */
                                *tar_ptr++ = *src_ptr++;
                                *tar_ptr++ = *src_ptr++;
                                *tar_ptr++ = *src_ptr;

                                /* Alpha channel data buffer avalable? */
                                if(src_data_alpha != NULL)
                                {
                                    src_ptr = &src_data_alpha[
                                        (y * src_bpl_alpha) +
                                        (x * (1 * sizeof(guint8)))
                                    ];
                                    /* Copy alpha of this pixel (1 byte). */
                                    *tar_ptr = *src_ptr;
                                }
                                else
                                {
                                    /* Set default alpha value. */
                                    *tar_ptr = 0xff;
                                }
                            }
                        }
                    }

		    /* Resize image viewer? */
		    if((w2 != NULL) && resize_on_load)
		    {
			gtk_widget_set_usize(
			    w2, width, height
			);
                        gtk_widget_queue_resize(w2);
			/* Notify toplevel to resize. */
			if(w != NULL)
			    gtk_widget_queue_resize(w);
		    }
		}	/* Loaded Imlib image data valid? */

		/* Unref Imlib image, it is no longer needed. */
		Imlib_destroy_image(imlib_handle, imlib_image);
		imlib_image = NULL;

                /* Load new image to image viewer, any currently loaded
                 * image will be unloaded first.
                 */
                ImgViewLoad(
                    iv,
                    width, height,
                    0,			/* Auto calculate bpl. */
                    IMGVIEW_FORMAT_RGBA,
                    (const guint8 *)data
                );

		/* Deallocate local copy of image data (if any). */
		g_free(data);
		data = NULL;
	    }
	}
#endif	/* HAVE_IMLIB */
#ifdef HAVE_IMLIB2





#endif  /* HAVE_IMLIB2 */

#endif	/* INCLUDE_IV_TITLE_IMAGE */
}
